/****************************************************************/
/*                                                              */
/*          Advanced Navigation Packet Protocol Library         */
/*       C Language Static Air Data Unit SDK, Version 1.0       */
/*             Copyright 2016, Advanced Navigation              */
/*                                                              */
/****************************************************************/
/*
 * Copyright (C) 2016 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdint.h>
#include <string.h>
#include "an_packet_protocol.h"
#include "air_data_unit_packets.h"
 
/*
 * This file contains functions to decode and encode packets
 *
 * Decode functions take an an_packet_t and turn it into a type specific
 * to that packet so that the fields can be conveniently accessed. Decode
 * functions return 0 for success and 1 for failure. Decode functions are
 * used when receiving packets.
 *
 * Example decode
 *
 * an_packet_t an_packet
 * acknowledge_packet_t acknowledge_packet
 * ...
 * decode_acknowledge_packet(&acknowledge_packet, &an_packet);
 * printf("acknowledge id %d with result %d\n", acknowledge_packet.packet_id, acknowledge_packet.acknowledge_result);
 *
 * Encode functions take a type specific structure and turn it into an
 * an_packet_t. Encode functions are used when sending packets.
 *
 * Example encode
 *
 * an_packet_t an_packet;
 * boot_mode_packet_t boot_mode_packet;
 * ...
 * boot_mode_packet.boot_mode = boot_mode_bootloader;
 * encode_boot_mode_packet(&an_packet, &boot_mode_packet);
 * serial_port_transmit(an_packet_pointer(&an_packet), an_packet_size(&an_packet));
 *
 */

int decode_acknowledge_packet(acknowledge_packet_t *acknowledge_packet, an_packet_t *an_packet)
{
	if(an_packet->id == packet_id_acknowledge && an_packet->length == 4)
	{
		acknowledge_packet->packet_id = an_packet->data[0];
		memcpy(&acknowledge_packet->packet_crc, &an_packet->data[1], sizeof(uint16_t));
		acknowledge_packet->acknowledge_result = an_packet->data[3];
		return 0;
	}
	else return 1;
}

void encode_request_packet(an_packet_t *an_packet, uint8_t requested_packet_id)
{
	an_packet->id = packet_id_request;
	an_packet->length = 1;
	an_packet->data[0] = requested_packet_id;
}

int decode_boot_mode_packet(boot_mode_packet_t *boot_mode_packet, an_packet_t *an_packet)
{
	if(an_packet->id == packet_id_boot_mode && an_packet->length == 1)
	{
		boot_mode_packet->boot_mode = an_packet->data[0];
		return 0;
	}
	else return 1;
}

void encode_boot_mode_packet(an_packet_t *an_packet, boot_mode_packet_t *boot_mode_packet)
{
	an_packet->id = packet_id_boot_mode;
	an_packet->length = 1;
	an_packet->data[0] = boot_mode_packet->boot_mode;
}

int decode_device_information_packet(device_information_packet_t *device_information_packet, an_packet_t *an_packet)
{
	if(an_packet->id == packet_id_device_information && an_packet->length == 24)
	{
		memcpy(&device_information_packet->software_version, &an_packet->data[0], sizeof(uint32_t));
		memcpy(&device_information_packet->device_id, &an_packet->data[4], sizeof(uint32_t));
		memcpy(&device_information_packet->hardware_revision, &an_packet->data[8], sizeof(uint32_t));
		memcpy(&device_information_packet->serial_number[0], &an_packet->data[12], 3*sizeof(uint32_t));
		return 0;
	}
	else return 1;
}

void encode_restore_factory_settings_packet(an_packet_t *an_packet)
{
	uint32_t verification = 0x85429E1C;
	an_packet->id = packet_id_restore_factory_settings;
	an_packet->length = 4;
	memcpy(&an_packet->data[0], &verification, sizeof(uint32_t));
}

void encode_reset_packet(an_packet_t *an_packet)
{
	uint32_t verification = 0x21057A7E;
	an_packet->id = packet_id_reset;
	an_packet->length = 4;
	memcpy(&an_packet->data[0], &verification, sizeof(uint32_t));
}

/* State packets */

int decode_raw_sensors_packet(raw_sensors_packet_t *raw_sensors_packet, an_packet_t *an_packet)
{
	if(an_packet->id == packet_id_raw_sensors && (an_packet->length == 9 || an_packet->length == 13))
	{
		memcpy(&raw_sensors_packet->absolute_pressure, &an_packet->data[0], sizeof(float));
		memcpy(&raw_sensors_packet->differential_pressure, &an_packet->data[4], sizeof(float));
		raw_sensors_packet->status.r = an_packet->data[8];
		if(an_packet->length == 13)
		{
		    memcpy(&raw_sensors_packet->temperature, &an_packet->data[9], sizeof(float));
		}
		else
		{
		    raw_sensors_packet->temperature = 0;
		}
		return 0;
	}
	else return 1;
}

int decode_air_data_packet(air_data_packet_t *air_data_packet, an_packet_t *an_packet)
{
	if(an_packet->id == packet_id_air_data && an_packet->length == 25)
	{
		memcpy(&air_data_packet->altitude_delay, &an_packet->data[0], sizeof(float));
		memcpy(&air_data_packet->airspeed_delay, &an_packet->data[4], sizeof(float));
		memcpy(&air_data_packet->altitude, &an_packet->data[8], sizeof(float));
		memcpy(&air_data_packet->airspeed, &an_packet->data[12], sizeof(float));
		memcpy(&air_data_packet->altitude_standard_deviation, &an_packet->data[16], sizeof(float));
		memcpy(&air_data_packet->airspeed_standard_deviation, &an_packet->data[20], sizeof(float));
		air_data_packet->status.r = an_packet->data[24];
		return 0;
	}
	else return 1;
}
